// -*- C++ -*-
/*!
 * @file  FireWebCamera.h
 * @brief Capture Image from PointGray IEEE1394 Camera and Web Camera
 * @date  $Date$
 *
 * $Id$
 */

#ifndef FIREWEBCAMERA_H
#define FIREWEBCAMERA_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
#include "ImageInfoServiceSVC_impl.h"
#include "CameraInfoServiceSVC_impl.h"

#include "imagecaptureWebIEEE1394Cam.h" // for camera
#include "ImageViewer.h"  // for showing window

// Time stamp
#include "TimeStampUtil_forRTC.h"

#include <string>
#include <iostream>

using namespace ALTH;
using namespace RTC;

/*!
 * @class FireWebCamera
 * @brief Capture Image from PointGray IEEE1394 Camera and Web Camera
 *
 */
class FireWebCamera
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  FireWebCamera(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~FireWebCamera();

  /**
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  /*!
   * select Web/IEEE1394 camera
   * - Name: CameraModels CameraModels
   * - DefaultValue: WebCamera
   * - Constraint: WebCamera, IEEE1394Camera, IEEE1394StereoCamera
   */
  std::string m_CameraModels;

  /*!
   * select pixel format of image
   * - Name:  PixelFormat
   * - DefaultValue: RGB
   * - Constraint: RGB, BGR
   */
  std::string m_PixelFormat;

  /*!
   * select show image or not
   * - Name: ImageWindow
   * - DefaultValue: off
   * - Constraint: on, off
   */
  std::string m_ImageWindow;

  // DataOutPort declaration
  // Image data
  TimedOctetSeq m_ImageData;
  OutPort<TimedOctetSeq> m_ImageDataOut;
  // Right Image data
  TimedOctetSeq m_ImageDataR; // for right camera
  OutPort<TimedOctetSeq> m_ImageDataROut; // for right camera
  
  // CORBA Port declaration
  // this port provide "ImageInfoService" and "CameraInfoService".
  RTC::CorbaPort m_CamInfoPort;
  RTC::CorbaPort m_CamInfoRPort;
  RTC::CorbaPort m_StereoInfoPort; // for stereo camera

  // Service declaration
  // ImageInfoService interface
  ImageInfoServiceSVC_impl m_ImageInfoService;
  ImageInfoServiceSVC_impl m_ImageInfoServiceR; // for right camera
  // CameraInfoService interface
  CameraInfoServiceSVC_impl m_CameraInfoService;
  CameraInfoServiceSVC_impl m_CameraInfoServiceR; // for right camera
  // StereoCameraInfoService interface
  StereoCameraInfoServiceSVC_impl m_StereoInfoService; // for stereo camera

 private:
  // camera class(core logic)
  WebIEEE1394Cam _camera;

  // window to show captured image (OpenCV implement)
  ImageViewer _imgViewer[2];
  
};


extern "C"
{
  void FireWebCameraInit(RTC::Manager* manager);
};

#endif // FIREWEBCAMERA_H
